/*
   This is my slightly modified, commented version of
      Example 3-7 : Robot Arm: robot.c
   from
     OpenGL Programming Guide
       Chapter 3, Viewing
         Examples of Composing Several Transformations
           Building an Articulated Robot Arm

   http://www.glprogramming.com/red/chapter03.html#name8
*/
#include <windows.h>
#include "glut.h"

static int shoulder = 0, elbow = 0;

void init(void)
{
  glClearColor (0.0, 0.0, 0.0, 0.0);
  glShadeModel (GL_FLAT);
}

void display(void)
{
   glClear (GL_COLOR_BUFFER_BIT);
   glPushMatrix();

   // transformation 1; a rotation about the point (-1, 0, 0)
   glTranslatef (-1.0, 0.0, 0.0);
   glRotatef ((GLfloat) shoulder, 0.0, 0.0, 1.0);
   glTranslatef (1.0, 0.0, 0.0);

   glPushMatrix();  // object 1; the upper arm
     glScalef (2.0, 0.4, 1.0); // create a long narrow "beam" that is
     glutWireCube (1.0);       // centered at the origin
   glPopMatrix();

   // transformation 2; move to the end of the upper arm, then rotate
   glTranslatef (1.0, 0.0, 0.0);  //(notice that the upper arm has length 2, but it's centered at the origin)
   glRotatef ((GLfloat) elbow, 0.0, 0.0, 1.0);

   glPushMatrix();  // object 2; the lower arm
     glTranslatef (1.0, 0.0, 0.0); // move the center of the "beam" away from the end the upper arm
     glScalef (2.0, 0.4, 1.0); // create a long narrow "beam" that is
     glutWireCube (1.0);       // centered at the origin
   glPopMatrix();

   glPopMatrix();
   glutSwapBuffers();
}

void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluPerspective(65.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef (0.0, 0.0, -5.0);
}

void keyboard (unsigned char key, int x, int y)
{
   switch (key) {
      case 's':   /*  s key rotates at shoulder  */
         shoulder = (shoulder + 5) % 360;
         glutPostRedisplay();
         break;
      case 'S':
         shoulder = (shoulder - 5) % 360;
         glutPostRedisplay();
         break;
      case 'e':  /*  e key rotates at elbow  */
         elbow = (elbow + 5) % 360;
         glutPostRedisplay();
         break;
      case 'E':
         elbow = (elbow - 5) % 360;
         glutPostRedisplay();
         break;
      default:
         break;
   }
}

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize (500, 500);
   glutInitWindowPosition (100, 100);
   glutCreateWindow (argv[0]);
   init ();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutMainLoop();
   return 0;
}