/*
   This program draws an approximation of the snowflake
   in Figure 5.4 on page 193 of our textbook.
*/
#include <windows.h>
#include <math.h>
#include "glut.h"

#define PI 3.14159265

/* Prototype for the display callback function */
void display( void );


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(500,500);    /* 500 x 500 pixel window */
   glutInitWindowPosition(50,50);  /* place window near top left of display */
   glutCreateWindow(argv[0]);      /* window title */
   glutDisplayFunc(display);       /* display callback invoked when window opened */

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-10.0, 10.0, -10.0, 10.0); /* viewing volume */

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


void model_1( void )
{
   /* Here are the primitives that make up our basic model. */
   /* This is one "edge" of the snowflake. */
   glBegin(GL_LINE_STRIP);
     glVertex2f(10.0,  0.0);
     glVertex2f(9.0, sin(PI/6.0));

     glVertex2f(8.0, sin(PI/6.0));
     glVertex2f(8.5, 0.5+sin(PI/6.0));
     glVertex2f(8.0, 0.5+sin(PI/6.0));
     glVertex2f(7.5, sin(PI/6.0));

     glVertex2f(6.0, sin(PI/6.0));
     glVertex2f(6.5, 1.0+sin(PI/6.0));
     glVertex2f(6.0, 1.0+sin(PI/6.0));
     glVertex2f(5.5, sin(PI/6.0));

     glVertex2f(3.5, sin(PI/6.0));
     glVertex2f(5.0, 2.0+sin(PI/6.0));
     glVertex2f(4.5, 2.0+sin(PI/6.0));
     glVertex2f(3.0, sin(PI/6.0));

     glVertex2f(cos(PI/6.0), sin(PI/6.0));
   glEnd();
}//model_1()


void model_2( void )
{
   /* This draws one "arm" of the snowflake. */
   glPushMatrix();
     model_1();
     glScalef(-1.0, 1.0, 1.0);  // relect the edge across the x-axis
     model_1();
   glPopMatrix();
}//model_2()


void model_3( void )
{
   int i;
   glPushMatrix();
     /* This draws six "arms" of the snowflake. */
     for (i = 0; i < 6; i++)
     {
        glRotatef(60.0, 0.0, 0.0, 1.0);
        model_2();
     }
   glPopMatrix();
}//model_3()


/* The display() callback function */
void display( void )
{
   glClear(GL_COLOR_BUFFER_BIT); /* clear the window */

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glColor3f(1.0, 0.0, 0.0);     /* draw in red */

   model_3();                    /* draw our basic model */

   glFlush(); /* clear buffers */

}//display()
