/*
   This program draws an approximation of the snowflake
   in Figure 5.4 on page 193 of our textbook.
*/
#include <windows.h>
#include <iostream>
#include <fstream>
#include "glut.h"
#include "glui.h"

using namespace std;

#define PI 3.14159265

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

#define sliderInit 0.0
float sliderValue = sliderInit;  // this is set by the glui slider() callback function

/* Prototypes for the GLUT callback functions */
void display( void );
void reshape(int, int);
void slider(float);

void drawGRSFile(char *fileName)
{
   fstream inStream;
   inStream.open(fileName, ios::in);  // open the GRS file
   if(inStream.fail()) {cout << "can't open GRS file!"; return;};

   // read the comment lines
   char str[80];
   inStream.getline(str, 80, '\n');
   while (str[0] != '*')
      inStream.getline(str, 80, '\n');

   // read the figure extents
   GLfloat left, top, right, bottom;
   inStream >> left >> top >> right >> bottom;

   glBegin(GL_LINE_STRIP); // draw a "bounding box"
     glVertex2f(0, 0);
     glVertex2f(right, 0);
     glVertex2f(right, top);
     glVertex2f(0, top);
     glVertex2f(0, 0);
   glEnd();

   GLint numpolys;
   inStream >> numpolys;              // read the number of polylines
   for(int j = 0; j < numpolys; j++)  // read each polyline
   {
      GLint numLines;
      inStream >> numLines;           // read the number vertices in this polyline
      GLfloat x ,y;
      glBegin(GL_LINE_STRIP);         // draw the polyline
        for (int i = 0; i < numLines; i++)
        {
           inStream >> x >> y;        // read the next world coordinate pair
           glVertex2f(x, y);
        }
      glEnd();
   }
   inStream.close();
}//drawGRSFile()

int main(int argc, char** argv)
{
   int window;

   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(600,600);    /* 600 x 600 pixel window */
   glutInitWindowPosition(50,50);  /* place window near top left of display */
   window = glutCreateWindow(argv[0]);
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-5.0, 5.0, -5.0, 5.0); /* viewing volume */

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10, 0, -10, 20, sliderInit, slider);

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* The display() callback function */
void display( void )
{
   glClear(GL_COLOR_BUFFER_BIT); /* clear the window */

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glColor3f(1.0, 0.0, 0.0);     /* draw in red */
   drawGRSFile("dragon.grs");

   glColor3f(0.0, 0.0, 1.0);     /* draw in blue */

   /* Try many different combinations of translations, rotations, and scalings. */

//   glTranslatef(sliderValue*3.0, 0.0, 0.0);
//   glScalef(sliderValue*5.0, sliderValue*5.0, 1.0);
   glRotatef(sliderValue*45.0, 0.0, 0.0, 1.0);

   drawGRSFile("dragon.grs");

   glutSwapBuffers();  // glui needs double buffering!
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback function */
void slider(float value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()