/*
   This program draws several instances of our "basic model".
   It uses transformations to place the model at different
   places and orientations within world coordinates.

   This version uses glPushMatrix() and glPopMatrix() between
   model instances instead of glLoadIdentity().
*/
#include <windows.h>
#include "glut.h"


/* Prototype for the display callback function */
void display( void );


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(500,500);    /* 500 x 500 pixel window */
   glutInitWindowPosition(50,50);  /* place window near top left of display */
   glutCreateWindow("Scene");      /* window title */
   glutDisplayFunc(display);       /* display callback invoked when window opened */

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-10.0, 10.0, -10.0, 10.0); /* viewing volume */

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* This function sends one instance of our model down the graphics pipeline. */
void model( void )
{
   /* Here are the primitives that make up our basic model. */
   glBegin(GL_LINE_LOOP);
     glVertex2f( 1.0,  1.0);  // a square
     glVertex2f(-1.0,  1.0);
     glVertex2f(-1.0, -1.0);
     glVertex2f( 1.0, -1.0);
   glEnd();
   glBegin(GL_LINE_LOOP);
     glVertex2f( 2.0,  2.0);  // another square
     glVertex2f(-2.0,  2.0);
     glVertex2f(-2.0, -2.0);
     glVertex2f( 2.0, -2.0);
   glEnd();
   glBegin(GL_LINES);
     glVertex2f( 1.0,  1.0);  // two line segments
     glVertex2f( 2.0,  2.0);
     glVertex2f(-2.0,  1.0);
     glVertex2f( 1.0, -2.0);
   glEnd();
}//model()


/* The display() callback function */
void display( void )
{
   glClear(GL_COLOR_BUFFER_BIT);  /* clear the window */

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   /* Draw four instances of our basic model. */

   glColor3f(1.0, 0.0, 0.0);         /* draw in red */
   glPushMatrix();
     glTranslatef(3.0, 3.0, 0.0);
     model();
   glPopMatrix();

   glColor3f(0.0, 1.0, 0.0);         /* draw in green */
   glPushMatrix();
     glTranslatef(-3.0, 0.0, 0.0);
     model();
   glPopMatrix();

   glColor3f(0.0, 0.0, 1.0);         /* draw in blue */
   glPushMatrix();
     glTranslatef(-5.0, -5.0, 0.0);
     glRotatef(45.0, 0.0, 0.0, 1.0);
     glScalef(0.5, 0.5, 1.0);
     model();
   glPopMatrix();

   glColor3f(0.0, 0.0, 0.0);         /* draw in black */
   glPushMatrix();
     glRotatef(-45.0, 0.0, 0.0, 1.0);
     glScalef(0.5, 0.5, 1.0);
     glTranslatef(5.0, 5.0, 0.0);
     model();
   glPopMatrix();

   glFlush(); /* clear buffers */

   /* Question: What happens if you comment out all of the
                matrix pushes and pops? Be sure to cause a
                re-display event.
   */

}//display()
