/*
   This version shows how, when reading transformations
   from top to bottom (in the code), we can think of each
   transformation as transforming a "coordinate frame".

   This version does a rotation followed by a translation
   (when read from top to bottom).
*/
#include <windows.h>
#include "glut.h"


/* Prototype for the display callback function */
void display( void );


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(500,500);    /* 500 x 500 pixel window */
   glutInitWindowPosition(50,50);  /* place window near top left of display */
   glutCreateWindow("Page 29 v2"); /* window title */
   glutDisplayFunc(display);       /* display callback invoked when window opened */

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-3.0, 3.0, -3.0, 3.0); /* viewing volume */

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* This function sends one instance of the model down the graphics pipeline. */
void model( void )
{
   glBegin(GL_LINE_LOOP);
      glVertex2f( 0.5,  0.0);
      glVertex2f( 0.0,  1.0);
      glVertex2f(-0.5,  0.0);
      glVertex2f(-0.5, -0.4);
      glVertex2f( 0.5, -0.4);
   glEnd();
}//model()


void frame(float gray)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glColor3f(gray, gray, gray); // Draw in gray.
   glBegin(GL_LINES);           // Draw a pair of axes.
      glVertex2f(0.0, 0.0);
      glVertex2f(1.0, 0.0);
      glVertex2f(0.0, 0.0);
      glVertex2f(0.0, 1.0);
   glEnd();
   glPopAttrib();
}//frame()


/* The display() callback function */
void display( void )
{
   glClear(GL_COLOR_BUFFER_BIT);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glColor3f(0.7, 0.7, 1.0);     // Draw in light blue.
   model();                      // Draw the basic model.
   glColor3f(0.0, 0.0, 1.0);     // Draw in blue.

   frame(0.7);                      // Show the original "coordinate frame" (in light gray).
   glRotatef(-45.0, 0.0, 0.0, 1.0); // This is the code that is in the book.
   frame(0.5);                      // Show the transformed "coordinate frame".
   glTranslatef(1.0, 1.0, 0.0);
   frame(0.0);                      // Show the transformed "coordinate frame".
   model();                         // Show the transformed model.

   glFlush();

}//display()
